#ifndef __DVBCHAIN_H
#define __DVBCHAIN_H

#include <vdr/channels.h>
#include <vdr/player.h>
#include <vdr/receiver.h>
#include <vdr/ringbuffer.h>
#include <vdr/thread.h>
#include <vdr/tools.h>
#include "dvbfilter.h"
#include "dvbsource.h"

// --- cRecorderChain --------------------------------------------------------

class cRecorderChain : public cListObject
                     , private cReceiver
                     , private cThread
{
public:
    static cRecorderChain* Create(cDevice* Device, cTimer* Timer);
    //static void StopRecording() { delete currentRecorder; currentRecorder = NULL; }

    //static bool IsRecording() { return currentRecorder != NULL; }

private:
    // cReceiver Implementation
    virtual void Activate(bool On) { if (On) Start(); else Cancel(3); }
    virtual void Receive(uchar* Data, int Length);

private:
    // cThread Implementation
    virtual void Action();

private:
    cRecorderChain(char const* FileName, cChannel const* Channel, int Priority);
    ~cRecorderChain() { Detach(); }

    cString fileName;
    cRingBufferLinear buffer;
    cExternalSource<stDvbTS> tsSource;
    cTSPacketScanner tsScanner;
    cTSPayloadExtractor vEsExtractor;
    cESVideoIndexer vEsIndexer;
    cDvbFileWriter<stDvbTS> fileWriter;
};

// --- nsRecorderChains ------------------------------------------------------

namespace nsRecorderChains
{
    bool Start(cTimer* Timer = NULL);
    
};

// --- cPlayerChain ----------------------------------------------------------

class cPlayerChain : public cDvbSink<stDvbPES> // XXX
                   , private cPlayer
                   , private cThread
                   , public cControl
{
public:
    cPlayerChain(char const* FileName);

private:
    // cDvbSink Implementation
    virtual void ReceivePackets(uchar const* Data, int Length);

private:
    // cPlayer Implementation
    virtual void Activate(bool On) { if (On) Start(); else Cancel(3); }

private:
    // cThread Implementation
    virtual void Action();

private:
    // cControl Implementation
    virtual void Hide() {}
    virtual eOSState ProcessKey(eKeys Key) { return osContinue; }

private:
    cFileName fileName;
    cIndexFile index;
    cUnbufferedFile* recordFile;
    cExternalSource<stDvbTS> tsSource;
    cTSPacketScanner tsScanner;
    cTS2PESRemuxer pesRemuxer;
};

#endif // __DVBCHAIN_H
