/*
 *  dvbfilter.c
 *  $Id$
 */

#include "dvbfilter.h"
#include "tstools.h"

// --- cTSPacketScanner ------------------------------------------------------

void cTSPacketScanner::ReceivePackets(uchar const* Data, int Length)
{
    uchar const* end = Data + Length;
    for (uchar const* p = Data; p < end; p += TS_SIZE) {
        uchar pid = GetTSPid(p);

        if (p[1] & TS_ERROR > 0)
            ++tsErrors;

        if ((p[3] & PAY_LOAD) && (p[3] & CONT_CNT_MASK) != ccCounter[pid]) {
            if (ccCounter[pid] != 0 && (p[3] & CONT_CNT_MASK) != ccCounter[pid] + 1)
                ++ccErrors;
            ccCounter[pid] = p[3] & CONT_CNT_MASK;
        }

        EmitPackets(p, TS_SIZE);

        PacketInfos().PacketOffset += TS_SIZE;
    }
}

// --- cTSPayloadExtractor ---------------------------------------------------

void cTSPayloadExtractor::ReceivePackets(uchar const* Data, int Length)
{
    uchar const* end = Data + Length;
    for (uchar const* p = Data; p < end; p += TS_SIZE) {
        if (GetTSPid(p) != pid)
            continue;

        if (!(p[3] & PAY_LOAD)) // only dump packets that contain payload
            continue;

        size_t payloadOffset = 4;
        if (p[3] & ADAPT_FIELD) { // packet has adaptation field
            payloadOffset += p[4] + 1;
            if (payloadOffset > TS_SIZE - 1)
                continue;
        }

        size_t length = TS_SIZE - payloadOffset;
        EmitPackets(p + payloadOffset, length);
    }
}

// --- cTS2PESRemuxer --------------------------------------------------------

void cTS2PESRemuxer::ReceivePackets(uchar const* Data, int Length)
{
    int tries = 2;
    while (Length > 0 && tries-- > 0) {
        int Put = remux.Put(Data, Length);
        Length -= Put;
        Data += Put;

        int Count;
        uchar const* p = remux.Get(Count);
        if (p != NULL) {
            EmitPackets(p, Count);
            remux.Del(Count);
        }
    }

    if (Length > 0)
        esyslog("ERROR: ts2pes remux buffer overflow, %d bytes dropped", Length);
}
