/*
 *  dvbfilter.h
 *  $Id$
 */

#ifndef __DVBFILTER_H
#define __DVBFILTER_H

#include "dvbsink.h"
#include "dvbsource.h"

// --- cDvbFilter ------------------------------------------------------------

template<class Input, class Output>
class cDvbFilter : public cDvbSink<Input>
                 , public cDvbSource<Output>
{
protected:
    cDvbFilter() {}

private:
    // cDvbSource Implementation
    virtual tDvbPacketInfos& GetPacketInfos() { return cDvbSink<Input>::PacketInfos(); }
};

// --- cTSPacketScanner ------------------------------------------------------

class cTSPacketScanner : public cDvbFilter<stDvbTS, stDvbTS>
    ///< This provides a filter that receives raw TS data and guarantees to 
    ///< emit only ranges of full TS packets, with the first byte always being
    ///< 0x47 and the length always being a multiple of 188
{
public:
    cTSPacketScanner();

private:
    // cDvbSink Implementation
    virtual void ReceivePackets(uchar const* Data, int Length);
    
    uchar ccCounter[1 << 13]; // XXX
    size_t ccErrors;
    size_t tsErrors;
};

inline cTSPacketScanner::cTSPacketScanner()
    : ccErrors(0)
    , tsErrors(0)
{
    memset(ccCounter, static_cast<uchar>(-1), sizeof(ccCounter));
}

// --- cTSPayloadExtractor ---------------------------------------------------

class cTSPayloadExtractor : public cDvbFilter<stDvbTS, stDvbES>
    ///< This provides a filter that receives ranges of full TS packets and
    ///< emits the payload of all packets that match a given pid.
{
public:
    cTSPayloadExtractor(int Pid);

private:
    // cDvbSink Implementation
    virtual void ReceivePackets(uchar const* Data, int Length);

private:
    int pid;
};

inline cTSPayloadExtractor::cTSPayloadExtractor(int Pid)
    : pid(Pid)
{
}

// --- cTS2PESRemuxer --------------------------------------------------------

class cTS2PESRemuxer : public cDvbFilter<stDvbTS, stDvbPES>
{
public:
    cTS2PESRemuxer(int VPid, int const* APids, int const* DPids);

private:
    // cDvbSink Implementation
    virtual void ReceivePackets(uchar const* Data, int Length);

private:
    cRemux remux;
};

inline cTS2PESRemuxer::cTS2PESRemuxer(int VPid, int const* APids, int const* DPids)
    : remux(VPid, APids, DPids, NULL, false)
{
}

#endif // __DVBFILTER_H
