#ifndef __DVBSINK_H
#define __DVBSINK_H

#include <vdr/recording.h>
#include <vdr/remux.h>
#include <vdr/tools.h>
#include "dvbpacket.h"
#include "streamtypes.h"

template<class Output>
class cDvbSource;

// --- cDvbSink --------------------------------------------------------------

template<class Input>
class cDvbSink : public cListObject
{
    friend class cDvbSource<Input>;

protected:
    cDvbSink();
	virtual ~cDvbSink() {}

    tDvbPacketInfos& PacketInfos() { return *infos; }

private:
    void SetDvbPacketInfos(tDvbPacketInfos& Infos) { infos = &Infos; }

	virtual void ReceivePackets(uchar const* Data, int Length) = 0;
        ///< Gets called by a chain feeder or by a sink when packets are being
        ///< transferred to this sink. 

    tDvbPacketInfos* infos;
};

template<class Input>
inline cDvbSink<Input>::cDvbSink()
    : infos(NULL)
{
}

// --- cESVideoIndexer -------------------------------------------------------

class cESVideoIndexer :  public cDvbSink<stDvbES>
{
public:
    cESVideoIndexer(char const* FileName);

private:
    // cDvbSink Implementation
    virtual void ReceivePackets(uchar const* Data, int Length);
        
private:
    size_t found;
    uint64_t packetStart;
    uchar lastStartCode;
    uint64_t pictureStart;
    uchar pictureType;
    cIndexFile indexFile;

    static const uint64_t InvalidOffset = ~(uint64_t)0;
};

inline cESVideoIndexer::cESVideoIndexer(char const* FileName)
    : found(0)
    , packetStart(0)
    , lastStartCode(0)
    , pictureStart(InvalidOffset)
    , pictureType(NO_PICTURE)
    , indexFile(FileName, true)
{
}

// --- cDvbFileWriter --------------------------------------------------------

template<class Input>
class cDvbFileWriter : public cDvbSink<Input>
{
public:
    cDvbFileWriter(char const* FileName);

private:
    // cDvbSink Implementation
    virtual void ReceivePackets(uchar const* Data, int Length);

private:
    cFileName fileName;
    cUnbufferedFile* recordFile;
};

template<class Input>
inline cDvbFileWriter<Input>::cDvbFileWriter(char const* FileName)
    : fileName(FileName, true)
    , recordFile(fileName.Open())
{
}

template<class Input>
void cDvbFileWriter<Input>::ReceivePackets(uchar const* Data, int Length)
{
    if (recordFile->Write(Data, Length) < 0)
        LOG_ERROR_STR(fileName.Name());
}

#endif // __DVBSINK_H
