#include <vdr/device.h>
#include "tstools.h"

uchar const* GetTSPacketRange(uchar const* Data, int& Length, int& Used)
{
    int Skipped = 0;
    while (Skipped < Length && Data[Skipped] != TS_SYNC_BYTE)
        ++Skipped;

    if (Skipped > 0)
        esyslog("ERROR: GetTSPacketRange skipped %d byte to sync on TS packet", Skipped);

    Used = Skipped;
    while (Used + TS_SIZE - 1 < Length && Data[Used] == TS_SYNC_BYTE)
        Used += TS_SIZE;

    Length = Used - Skipped;
    return Data + Skipped;
}

int GetTSPid(uchar const* Data)
{
    return ((Data[1] & PID_MASK_HI) << 8) | Data[2];
}

// --- cAsyncTSBuffer --------------------------------------------------------

void cAsyncTSBuffer::Put(uchar const* Data, int Length)
{
    if (!Running())
        return;

    int tries = 5;
    int put = 0;
    while (put < Length && --tries)
        put += buffer.Put(Data + put, Length - put);

    if (put < Length)
        buffer.ReportOverflow(Length - put);
}

void cAsyncTSBuffer::Action()
{
	while (Running()) {
        // XXX check for buffer fill level

		int length;
		uchar const* b = buffer.Get(length);
		if (!b) // no packets available
            continue;

        int used;
        uchar const* data = GetTSPacketRange(b, length, used);
        emitFunc(data, length, param);
        buffer.Del(used);
	}
}
