#ifndef __TSTOOLS_H
#define __TSTOOLS_H

#include <vdr/device.h>
#include <vdr/thread.h>
#include <vdr/tools.h>

#undef PID_MASK_HI
#undef CONT_CNT_MASK

// Transport Stream Flags:
uchar const PAY_LOAD    = 0x10;
uchar const ADAPT_FIELD = 0x20;
uchar const PAY_START   = 0x40;
uchar const TS_ERROR    = 0x80;

// Bitmasks:
uchar const PID_MASK_HI   = 0x1F;
uchar const CONT_CNT_MASK = 0x0F;
uchar const COD_TYPE_MASK = 0x38;

// Start Codes:
uchar const SC_SEQUENCE = 0xB3;  // "sequence header code"
uchar const SC_GROUP    = 0xB8;  // "group start code"
uchar const SC_PICTURE  = 0x00;  // "picture start code"

uchar const* GetTSPacketRange(uchar const* Data, int& Length, int& Used);
int GetTSPid(uchar const* Packet);

// --- cAsyncTSBuffer --------------------------------------------------------

class cAsyncTSBuffer : private cThread
{
public:
    typedef void (*EmitFunc)(uchar const* Data, int Length, void* Param);

    cAsyncTSBuffer(EmitFunc Func, void* Param, bool WriteMayBlock);

    using cThread::Start;
    using cThread::Cancel;

    void Put(uchar const* Data, int Length);

private:
    // cThread Implementation
    virtual void Action();

private:
    cRingBufferLinear buffer;
    EmitFunc emitFunc;
    void* param;
};

inline cAsyncTSBuffer::cAsyncTSBuffer(EmitFunc Func, void* Param, bool WriteMayBlock)
    : cThread("AsyncTSBuffer")
    , buffer(MEGABYTE(2), TS_SIZE * 2, !WriteMayBlock, "AsyncTSBuffer")
    , emitFunc(Func)
    , param(Param)
{
    buffer.SetTimeouts(WriteMayBlock ? 1000 : 0, 100);
}

#endif // __TSTOOLS_H
